#
# Copyright (C) 2002, 2003, 2004, 2007 Red Hat, Inc.
# Authors:
# Brent Fox <bfox@redhat.com>
# Thomas Woerner <twoerner@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

from fw_config import _
from fw_functions import getPortID, getServiceName

class _Service:
    def __init__ (self, key, name, ports, description=None, modules=[ ],
                  default=False):
        self.key = key
        self.name = name
        self.ports = ports
        self.description = description
        self.modules = modules
        self.default = default # uses no-option to disable

service_list = [
    _Service("ipp", _("Printing (IPP)"), [ ("631", "tcp"), ("631", "udp"), ],
             _("The Internet Printing Protocol (IPP) protocol is used for "
               "distributed printing. It provides the ability to get "
               "information about a printer (e.g. capability and status) "
               "and to control printer jobs. If you plan to use a network "
               "printer via cups, do not disable this option."),
             default=True),
    _Service("mdns", _("Multicast DNS (mDNS)"), [ ("5353", "udp"), ],
             _("mDNS provides the ability to do use DNS programming "
               "interfaces, packet formats and operating semantics, in a "
               "small network without a conventional DNS server. If you plan "
               "to use Avahi, do not disable this option."),
             default=True),
    _Service("ipsec", _("IPSec"), [ ("", "ah"), ("", "esp"), ],
             _("Internet Protocol Security (IPsec) incorporates security "
               "for network transmissions into the Internet Protocol (IP) "
               "directly. IPSec provides methods for both encrypting data "
               "and authentication the host or network it sends to. If you "
               "plan to use a vpnc server or FreeS/WAN, do not diable this "
               "option."),
             default=True),

    _Service("ssh", _("SSH"), [ ("22", "tcp"), ],
             _("Secure Shell (SSH) is a protocol for logging into and "
               "executing commands on remote machines. It provides secure "
               "encrypted communications. If you plan on accessing your "
               "machine remotely via SSH over a firewalled interface, enable "
               "this option. You need the openssh-server package installed "
               "for this option to be useful." )),
#    _Service("telnet", _("Telnet"), [ ("23", "tcp"), ],
#             "Telnet is a protocol for logging into remote machines. It "
#             "is unencrypted, and provides little security from network "
#             "snooping attacks. Enabling telnet is not recommended. You need "
#             "the telnet-server package installed for this option to be "
#             "useful."),
    _Service("http", _("WWW (HTTP)"), [ ("80", "tcp"), ],
             _("HTTP is the protocol used to serve Web pages. If you plan on "
               "making your Web server publicly available, enable this option. "
               "This option is not required for viewing pages locally or "
               "developing Web pages.")),
    _Service("ftp", _("FTP"), [ ("21", "tcp"), ],
             _("FTP is a protocol used for remote file transfer. If you plan "
               "on making your FTP server publicly available, enable this "
               "option. You need the vsftpd package installed for this option "
               "to be useful."),
             modules=[ "nf_conntrack_ftp", ]),
    _Service("nfs", _("NFS4"), [ ("nfs", "tcp"), ],
             _("This allowed you to use the NFS version 4 protocol via TCP "
               "to share files. You will need to have the NFS tools installed "
               "and properly configure your NFS server for this option to be "
               "useful.")),
    _Service("https", _("Secure WWW (HTTPS)"), [ ("https", "tcp"), ],
             _("HTTPS is a modified HTTP used to serve Web pages when security "
               "is important. Examples are sites that require logins like "
               "stores or web mail. This option is not required for viewing "
               "pages locally or developing Web pages. You need the httpd "
               "package installed for this option to be useful.")),
    _Service("smtp", _("Mail (SMTP)"), [ ("smtp", "tcp"), ],
             _("This allows incoming SMTP mail delivery. If you need to allow "
               "remote hosts to connect directly to your machine to deliver "
               "mail, enable this option. You do not need to enable this if "
               "you collect your mail from your ISP's server by POP3 or IMAP, "
               "or if you use a tool such as fetchmail. Note that an "
               "improperly configured SMTP server can allow remote machines "
               "to use your server to send spam.")),
    _Service("samba", _("Samba"),
             [ ("netbios-ns", "udp"), ("netbios-dgm", "udp"),
               ("netbios-ssn", "tcp"), ("microsoft-ds", "tcp"), ],
             _("This allows you to access and participate in Windows "
               "file and printer sharing networks.  You need the Samba "
               "package installed for this option to be useful."),
             modules=[ "nf_conntrack_netbios_ns", ]),

    _Service("domain", _("DNS"), [ ("53", "tcp"), ("53", "udp"), ],
             _("The Domain Name System (DNS) is used to provide and request "
               "host and domain names. Enable this option, if you plan to "
               "provide a domain name service (e.g. with bind).")),
    _Service("imaps", _("IMAP over SSL"), [ ("993", "tcp"), ],
             _("The Internet Message Access Protocol over SSL (IMAPs) allows "
               "a local client to access e-mail on a remote server in a secure "
               "way. If you plan to provide a IMAP over SSL service (e.g. with "
               "dovecot), enable this option.")),
    _Service("pop3s", _("POP-3 over SSL"), [ ("995", "tcp"), ],
             _("The Post Office Protocol version 3 (POP3) is a protocol to "
               "retrieve e-mail from a remote server over a TCP/IP "
               "connection. Enable this option, if you plan to provide a POP3 "
               "service (e.g. with dovecot).")),
    _Service("radius", _("RADIUS"), [ ("1812", "udp"), ("1813", "udp"), ],
             _("The Remote Authentication Dial In User Service (RADIUS) is a "
               "protocol for user authentication over networks. It is mostly "
               "used for modem, DSL or wireless user authentication. If you "
               "plan to provide a RADIUS service (e.g. with freeradius), "
               "enable this option.")),
    ]

def getByKey(key):
    for x in service_list:
        if x.key == key:
            return x
    return None

def getByName(name):
    for x in service_list:
        if x.name == name:
            return x
    return None

def getByPort(port, proto):
    for x in service_list:
        id = getPortID(port)
        name = getServiceName(port, proto)
        if (id, proto) in x.ports or (str(id), proto) in x.ports or \
               (name, proto) in x.ports:
            return x
    return None        

def getServicesAndPorts(config):
    services = [ ]
    other_ports = [ ]
    matched = [ ]
    for svc in service_list:
        if svc.default:
            continue
        _matched = [ ]
        if config.ports:
            all_matched = True
            for (port, proto) in svc.ports:
                id = getPortID(port)
                name = getServiceName(port, proto)                    
                if ((id,), proto) in config.ports:
                    _matched.append(((id,), proto))
                elif ((port,), proto) in config.ports:
                    _matched.append(((port,), proto))
                elif ((name,), proto) in config.ports:
                    _matched.append(((name,), proto))
                else:
                    all_matched = False
        else:
            all_matched = False
        if all_matched:
            services.append(svc.key)
            matched.extend(_matched)
        del _matched
    if config.ports:
        for entry in config.ports:
            if entry in matched:
                continue
            other_ports.append(entry)
    del matched
    return (services, other_ports)
